Package: LVGP
Type: Package
Title: Latent Variable Gaussian Process Modeling with Qualitative and
        Quantitative Input Variables
Version: 2.1.6
Authors@R: c(
    person("Siyu", "Tao", email = "siyutao2020@u.northwestern.edu", role = c("aut", "cre")),
    person("Yichi", "Zhang", role = "aut"),
    person("Daniel W.", "Apley", role = "aut"),
    person("Wei", "Chen", role = "aut")
    )
Description: Fit response surfaces for datasets with latent-variable Gaussian
    process modeling, predict responses for new inputs, and plot latent
    variables locations in the latent space (only 1D or 2D). The input
    variables of the datasets can be quantitative, qualitative/categorical or
    mixed. The output variable of the datasets is a scalar (quantitative). The
    optimization of the likelihood function is done using a successive
    approximation/relaxation algorithm similar to another GP modeling package
    "GPM". The modeling method is published in "A Latent Variable Approach to
    Gaussian Process Modeling with Qualitative and Quantitative Factors" by
    Yichi Zhang, Siyu Tao, Wei Chen, and Daniel W. Apley (2018)
    <doi:10.48550/arXiv.1806.07504>. The package is developed in IDEAL of
    Northwestern University.
License: GPL-2
Encoding: UTF-8
LazyData: true
Imports: lhs(>= 0.14), randtoolbox(>= 1.17)
Depends: R (>= 3.4.0), stats (>= 3.2.5), parallel (>= 3.2.5)
RoxygenNote: 6.1.1
NeedsCompilation: no
Packaged: 2026-07-21 14:50:57 UTC; sto8
Author: Siyu Tao [aut, cre],
  Yichi Zhang [aut],
  Daniel W. Apley [aut],
  Wei Chen [aut]
Maintainer: Siyu Tao <siyutao2020@u.northwestern.edu>
Repository: CRAN
Date/Publication: 2026-07-21 16:20:08 UTC

# Generated by roxygen2: do not edit by hand

export(LVGP_fit)
export(LVGP_plot)
export(LVGP_predict)
export(corr_mat)
export(neg_log_l)
export(to_latent)
import(graphics)
import(lhs)
import(parallel)
import(randtoolbox)

#' @title The Function for Constructing the Correlation Matrix in \code{LVGP} Package
#'
#' @description Builds the correlation matrix given two datasets, and the type and parameters of the correlation function.
#'
#' @param X1,X2 Matrices containing the data points. The rows and columns of both \code{X1} and \code{X2} denote individual observation settings and dimension, respectively.
#' @param phi_full The vector storing all the scale (aka roughness) parameters of the correlation function. See \code{reference 1}.
#'
#' @return R The Correlation matrix with size \code{nrow(X1)}-by-\code{nrow(X2)}. See \href{https://en.wikipedia.org/wiki/Correlation_matrix}{here}.
#'
#' @note This function is \strong{NOT} exported once the LVGP package is loaded.
#' @export
#'
#' @references
#' \enumerate{
#' \item "A Latent Variable Approach to Gaussian Process Modeling with Qualitative and Quantitative Factors", Yichi Zhang, Siyu Tao, Wei Chen, and Daniel W. Apley (\href{https://arxiv.org/abs/1806.07504}{arXiv})
#' }
#'
#' @seealso
#' \code{\link[LVGP]{LVGP_fit}} to see how a GP model can be fitted to a training dataset.\cr
#' \code{\link[LVGP]{LVGP_predict}} to use the fitted LVGP model for prediction.\cr
#' \code{\link[LVGP]{LVGP_plot}} to plot the features of the fitted model.
#'
#' @examples
#' # see the examples in the documentation of the function LVGP_fit.

corr_mat <-  function(X1, X2, phi_full){

  k = nrow(X1)
  p = ncol(X1)
  kk = nrow(X2)
  R = matrix(0, k, kk)
  phi_full = as.vector(phi_full)

  if (p != length(phi_full)){
    stop(paste('    Length of roughness parameters (', toString(length(phi_full)), ') does not match input dimension (',
               toString(p), ')!'))
  }

  if (k >= kk){
    for (i in 1: kk) {
      R[, i] = colSums((t(X1) - X2[i, ])^2*(10^phi_full))
    }
  } else{
    for (i in 1: k) {
      R[i , ] = colSums((t(X2) - X1[i, ])^2*(10^phi_full))
    }
  }

  R = exp(-0.5*R)

  return(R)
}

#' Dataset for the example in function 'LVGP_fit'
#'
#' Data are sampled from the modified math function based on the first example in the paper listed in \code{references}.
#' There are still 2 quantitative and 1 qualitative variables, but the qualitative variable has only 3 levels.
#' For each level, there are 8 training data points and 30 testing data points, all generated with Latin hypercube sampling.
#' In total, there are 24 training data points and 90 testing data points.
#'
#' @docType data
#'
#' @usage data(math_example)
#'
#' @format A named list containing training and test data:
#' \describe{
#'   \item{"X_tr"}{24-by-3 matrix for 24 training data inputs, 3rd column being the qualitative variable}
#'   \item{"Y_tr"}{24-by-1 matrix for 24 training data outputs}
#'   \item{"X_te"}{90-by-3 matrix for 90 testing data inputs, 3rd column being the qualitative variable}
#'   \item{"Y_te"}{90-by-1 matrix for 90 testing data outputs}
#' }
#'
#' @keywords example dataset
#'
#' @references
#' \enumerate{
#' \item "A Latent Variable Approach to Gaussian Process Modeling with Qualitative and Quantitative Factors", Yichi Zhang, Siyu Tao, Wei Chen, and Daniel W. Apley (\href{https://arxiv.org/abs/1806.07504}{arXiv})
#' }
#'
#' @source The dataset can be generated with the code at the end of this description file.
#'
#' @examples
#' data(math_example)
#' X_tr <- math_example$X_tr
#' Y_tr <- math_example$Y_tr
#' X_te <- math_example$X_te
#' Y_te <- math_example$Y_te
"math_example"
#
# n_lv <- 3 # number of levels for the qualitative variable
# lb <- c(0, 0); ub <- c(1, 1)
# n_tr_lv <- 8 # number of training samples for each level
# n_te_lv <- 30 # number of testing samples for each level
# n_tr <- n_lv*n_tr_lv # total number of training samples
# n_te <- n_lv*n_te_lv # total number of training samples
# X_tr <- matrix(0, nrow = n_tr, ncol = 3)
# Y_tr <- matrix(0, nrow = n_tr, ncol = 1)
# X_te <- matrix(0, nrow = n_te, ncol = 3)
# Y_te <- matrix(0, nrow = n_te, ncol = 1)
# var_fctr <- c(0, 12.0, 6.0)
# for (i in 1:n_lv) {
#   A <- lhs::maximinLHS(n_tr_lv, 2)
#   lhs_temp <- t(t(A)*(ub - lb) + lb)
#   X_tr[((i-1)*n_tr_lv+1):(i*n_tr_lv),1:2] <- lhs_temp
#   X_tr[((i-1)*n_tr_lv+1):(i*n_tr_lv),3] <- rep(i, n_tr_lv)
#   Y_tr[((i-1)*n_tr_lv+1):(i*n_tr_lv),] <- sin(2*pi*lhs_temp[,2]-pi) +
#     7*sin(2*pi*lhs_temp[,1]-pi)+var_fctr[i]*sin(2*pi*lhs_temp[,2]-pi)
#
#   A <- lhs::maximinLHS(n_te_lv, 2)
#   lhs_temp <- t(t(A)*(ub - lb) + lb)
#   X_te[((i-1)*n_te_lv+1):(i*n_te_lv),1:2] <- lhs_temp
#   X_te[((i-1)*n_te_lv+1):(i*n_te_lv),3] <- rep(i, n_te_lv)
#   Y_te[((i-1)*n_te_lv+1):(i*n_te_lv),] <- sin(2*pi*lhs_temp[,2]-pi) +
#     7*sin(2*pi*lhs_temp[,1]-pi)+var_fctr[i]*sin(2*pi*lhs_temp[,2]-pi)
# }
#
# model <- LVGP_fit(X_tr, Y_tr, ind_qual = c(3))
# output <- LVGP_predict(X_te, model)
# Y_hat <- output$Y_hat
# RRMSE <- sqrt(sum((Y_hat-Y_te)^2)/n_te)/(max(Y_te)-min(Y_te))
# LVGP_plot(model)
#
# math_example = list(X_tr=X_tr,Y_tr=Y_tr,X_te=X_te,Y_te=Y_te)

#' @title The Function for Transforming Qualitative/Categorical Variables into Latent Variables in \code{LVGP} Package
#'
#' @description Transforms qualitative/categorical variables into latent variables.
#'
#' @param X_qual Matrix or data frame containing (only) the qualitative/categorical data.
#' @param lvs_qual List containing levels of each qualitative variable
#' @param n_lvs_qual Number of levels of each qualitative variable
#' @param p_qual Number of qualitative variables
#' @param z_vec Latent variable parameters, i.e., latent variable values for each level of qualitative/categorical variables
#' @param dim_z Dimensionality of latent variables, usually 1 or 2
#' @param k Number of data points, equal to \code{nrow(X_qual)}
#'
#' @return Matrix containing transformed data
#'
#' @note This function is \strong{NOT} exported once the LVGP package is loaded.
#' @export
#'
#' @references
#' \enumerate{
#' \item "A Latent Variable Approach to Gaussian Process Modeling with Qualitative and Quantitative Factors", Yichi Zhang, Siyu Tao, Wei Chen, and Daniel W. Apley (\href{https://arxiv.org/abs/1806.07504}{arXiv})
#' }
#'
#' @seealso
#' \code{\link[LVGP]{LVGP_fit}} to see how a GP model can be fitted to a training dataset.\cr
#' \code{\link[LVGP]{LVGP_predict}} to use the fitted LVGP model for prediction.\cr
#' \code{\link[LVGP]{LVGP_plot}} to plot the features of the fitted model.
#'
#' @examples
#' # see the examples in the documentation of the function LVGP_fit.

to_latent <-  function(X_qual, lvs_qual, n_lvs_qual, p_qual, z_vec, dim_z, k){

  X_qual_la <- matrix(0, k, p_qual*dim_z)
  # note: the first levels of each variable are zeros in the latent space, no need to touch upon.
  ind_temp <- as.integer(0)
  for (i in 1:p_qual) {
    n_lvs <- n_lvs_qual[i]
    lvs <- lvs_qual[[i]]
    z_i <- z_vec[(dim_z*ind_temp+1) : (dim_z*(ind_temp+n_lvs-1))]
    ind_temp <- ind_temp+n_lvs-1
    for (j in 2:n_lvs) {
      mask <- X_qual[, i] == lvs[j]
      num_row <- sum(mask)
      if (num_row > 0) {
        X_qual_la[mask, ((i-1)*dim_z+1):(i*dim_z)] <-
          matrix(z_i[(dim_z*(j-2)+1):(dim_z*(j-1))], nrow = num_row, ncol = dim_z, byrow = TRUE)
      }

    }
  }
  return(X_qual_la)
}

#' @title The Fitting Function of \code{LVGP} Package
#'
#' @description Fits a latent-variable Gaussian process (LVGP) model to a dataset as described in \code{reference 1}.
#'     The input variables can be quantitative or qualitative/categorical or mixed.
#'     The output variable is quantitative and scalar.
#'
#' @param X Matrix or data frame containing the inputs of training data points. Each row is a data point.
#' @param Y Vector containing the outputs of training data points
#' @param ind_qual Vector containing the indices of columns of qualitative/categorical variables
#' @param dim_z Dimensionality of latent space, usually 1 or 2 but can be higher
#' @param eps The vector of smallest eigen values that the correlation matrix is allowed to have, which determines the nugget added to the correlation matrix.
#' @param lb_phi_ini,ub_phi_ini The initial lower and upper search bounds of the scale/roughness parameters (\code{phi}) of quantitative variables
#' @param lb_phi_lat,ub_phi_lat The later lower and upper search bounds of the scale/roughness parameters (\code{phi}) of quantitative variables
#' @param lb_z,ub_z The lower and upper search bounds of the latent parameters (\code{z}) of qualitative variables
#' @param n_opt The number of times the log-likelihood function is optimized
#' @param max_iter_ini The maximum number of iterations for each optimization run for largest (first) eps case
#' @param max_iter_lat The maximum number of iterations for each optimization run for after first eps cases
#' @param seed An integer for the random number generator. Use this to make the results reproducible.
#' @param progress The switch determining whether to print function run details
#' @param parallel The switch determining whether to use parallel computing
#' @param noise The switch for whether the data are assumed noisy
#'
#' @import lhs
#' @import randtoolbox
#' @import parallel
#'
#' @return A model of class `"LVGP model"`, a list with the following components:
#' \describe{
#' \item{quant_param}{A list containing the estimated parameter \code{phi} and its search bounds for quantitative variables}
#' \item{qual_param}{A list containing the estimated parameter \code{z} and its dimensionality, vectorized form and search bounds for qualitative variables}
#' \item{data}{A list containing the fitted dataset in verbose format}
#' \item{fit_detail}{A list of more detailed variables for fitting and prediction process}
#' \item{optim_hist}{Optimization history}
#' \item{setting}{Settings for the optimization and fitting process}
#' }
#'
#' @references
#' \enumerate{
#' \item "A Latent Variable Approach to Gaussian Process Modeling with Qualitative and Quantitative Factors", Yichi Zhang, Siyu Tao, Wei Chen, and Daniel W. Apley (\href{https://arxiv.org/abs/1806.07504}{arXiv})
#' }
#'
#' @export
#'
#' @seealso
#' \code{\link[stats]{optim}} for the details on \code{L-BFGS-B} algorithm used in optimization.\cr
#' \code{\link[LVGP]{LVGP_predict}} to use the fitted LVGP model for prediction.\cr
#' \code{\link[LVGP]{LVGP_plot}} to plot the features of the fitted model.
#'
#' @examples
#' # Math example with 2 quantitative and 1 qualitative variables (dataset included in the package):
#' #     Fit a model (with default settings) and evaluate the performance
#' #     by computing the root mean squared error (RMSE) in prediction.
#' #     Also, plot the latent variable parameters.
#' X_tr <- math_example$X_tr
#' Y_tr <- math_example$Y_tr
#' X_te <- math_example$X_te
#' Y_te <- math_example$Y_te
#' n_te <- nrow(X_te)
#' model <- LVGP_fit(X_tr, Y_tr, ind_qual = c(3))
#' output <- LVGP_predict(X_te, model)
#' Y_hat <- output$Y_hat
#' RRMSE <- sqrt(sum((Y_hat-Y_te)^2)/n_te)/(max(Y_te)-min(Y_te))
#' LVGP_plot(model)
#'

LVGP_fit <- function(X, Y, ind_qual = NULL, dim_z = 2, eps = 10^(seq(-1, -8, length.out = 15)),
                     lb_phi_ini = -2, ub_phi_ini = 2, lb_phi_lat = -8, ub_phi_lat = 3, lb_z = -3, ub_z = 3, n_opt = 8,
                     max_iter_ini = 100, max_iter_lat = 20, seed = 123, progress = FALSE,
                     parallel = FALSE, noise = FALSE) {

  set.seed(seed)

  ## function input checking & preprocessing
  if (progress){
    cat('** Checking & preprocessing the inputs...\n')
  }
  if (missing(X)){
    stop('    X must be provided.')
  }
  if (!is.matrix(X) && !is.data.frame(X)){
    stop('    X must be a matrix or a data frame.')
  }
  k <- nrow(X) # number of data points
  p_all <- ncol(X) # total number of input variables
  # check qualitative and quantitative parts of dataset
  if (is.null(ind_qual)) { # no qualitative/categorical variables
    p_qual <- as.integer(0)
    X_qual <- NULL
    X_quant <- X # data of quant variables
    if (!all(is.finite(X_quant)) || !is.numeric(X_quant)){
      stop('    All the elements of X_quant must be finite numbers.')
    }
    if (is.matrix(X_quant) == FALSE) {
      X_quant <- as.matrix(X_quant)
    }
    lvs_qual <- NULL
    n_lvs_qual <- NULL
    n_z <- 0
  } else {
    if (!all(is.finite(ind_qual)) || !is.numeric(ind_qual)){
      stop('    All the elements of ind_qual must be finite numbers.')
    }
    ind_qual <- as.vector(as.integer(ind_qual)) # indices of qual variables in X mat
    p_qual <- length(ind_qual) # number of qual variables
    X_qual <- X[, ind_qual] # data of qual variables
    if (is.data.frame(X_qual) == FALSE) {
      X_qual <- as.data.frame(X_qual)
    }
    if (p_qual == p_all) { # all the variables are qualitative/categorical
      X_quant <- NULL
    } else {
      X_quant <- X[, -ind_qual]
      if (!all(is.finite(X_quant)) || !is.numeric(X_quant)){
        stop('    All the elements of X_quant must be finite numbers.')
      }
      if (is.matrix(X_quant) == FALSE) {
        X_quant <- as.matrix(X_quant)
      }
    }

    lvs_qual <- list() # levels of each qual variable
    n_lvs_qual <- rep(0, p_qual) # number of levels of each qual variable
    for (i in 1:p_qual) {
      lvs_qual[[i]] <- sort(unique(X_qual[, i]))
      n_lvs_qual[i] <- length(lvs_qual[[i]])
    }
    n_z <- dim_z*(sum(n_lvs_qual)-p_qual) # number of latent parameters
  }


  if (missing(Y)){
    stop('    Y must be provided.')
  }
  if (!all(is.finite(Y)) || !is.numeric(Y)){
    stop('    All the elements of Y must be finite numbers.')
  }
  if (is.matrix(Y) == FALSE) {
    Y <- as.matrix(Y)
  }
  if (k != nrow(Y)){
    stop('    The number of rows (i.e., observations) in X and Y should match!')
  }

  dim_z <- as.integer(dim_z)
  if (dim_z != 1 && dim_z != 2) {
    warning('    The dimensionality of latent space is uncommon!')
  }

  eps <- as.numeric(eps)
  if (any(eps < (10^(round(log10(.Machine$double.eps))+3)))){
    stop(paste('    Increase the smallest member of eps. The minimum allowable is ', toString(10^(round(log10(.Machine$double.eps))+3))))
  }
  if (any(diff(eps) > 0)){
    stop('The elements of eps should be in a descending order.')
  }

  lb_phi_ini <- as.numeric(lb_phi_ini)
  ub_phi_ini <- as.numeric(ub_phi_ini)
  lb_phi_lat <- as.numeric(lb_phi_lat)
  ub_phi_lat <- as.numeric(ub_phi_lat)
  lb_z <- as.numeric(lb_z)
  ub_z <- as.numeric(ub_z)
  n_opt <- as.integer(n_opt) # number of starting points for optimization
  max_iter_ini <- as.integer(max_iter_ini)
  max_iter_lat <- as.integer(max_iter_lat)

  p_quant <- p_all - p_qual

  ## normalization of X_quant and Y
  if (p_quant > 0) {
    X_quant_min <- apply(X_quant, 2, min)
    X_quant_max <- apply(X_quant, 2, max)
    X_quant <- t((t(X_quant)-X_quant_min)/(X_quant_max-X_quant_min))
  } else {
    X_quant_min <- NULL
    X_quant_max <- NULL
  }

  Y_min <- apply(Y, 2, min)
  Y_max <- apply(Y, 2, max)
  Y <- t((t(Y)-Y_min)/(Y_max-Y_min))

  ## initiation for optimization
  n_hyper <- p_quant+n_z
  lb_ini <- c(rep(lb_phi_ini, p_quant), rep(lb_z, n_z))
  ub_ini <- c(rep(ub_phi_ini, p_quant), rep(ub_z, n_z))
  lb_lat <- c(rep(lb_phi_lat, p_quant), rep(lb_z, n_z))
  ub_lat <- c(rep(ub_phi_lat, p_quant), rep(ub_z, n_z))
  if (dim_z == 2 && p_qual != 0) {
    temp_ind <- p_quant
    for (i in 1:p_qual) {
      n_lvs <- n_lvs_qual[i]
      lb_ini[temp_ind+dim_z] <- -1e-4
      ub_ini[temp_ind+dim_z] <- 1e-4
      lb_lat[temp_ind+dim_z] <- -1e-4
      ub_lat[temp_ind+dim_z] <- 1e-4
      temp_ind <- temp_ind + dim_z*(n_lvs-1)
    }
  }

  opt_ctrl_ini <- c(trace=progress, maxit = max_iter_ini,  REPORT = 10, lmm = 100)
  opt_ctrl_lat <- c(trace=progress, maxit = max_iter_lat,  REPORT = 10, lmm = 100)

  setting <- list(max_iter_ini = max_iter_ini, max_iter_lat = max_iter_lat, seed = seed, n_opt = n_opt,
                  lb_phi_ini = lb_phi_ini, ub_phi_ini = ub_phi_ini,
                  lb_phi_lat = lb_phi_lat, ub_phi_lat = ub_phi_lat,
                  lb_z = lb_z, ub_z = ub_z, trace=progress,
                  parallel = parallel, eps = eps)

  start_time <- proc.time()[3]

  A <- lhs::maximinLHS(n_opt, n_hyper)
  hyper0 <- t(t(A)*(ub_ini - lb_ini) + lb_ini) # starting points (of hyperparameters) for optimization

  M <- matrix(1, nrow = k, ncol = 1)

  ## optimization runs
  # prepare parallel start
  if (parallel) {
    n_cores <- detectCores()
    optimfun <- function(x0_ele, objfun, addinput) {
      p_quant <- addinput$p_quant
      p_qual <- addinput$p_qual
      lvs_qual <- addinput$lvs_qual
      n_lvs_qual <- addinput$n_lvs_qual
      dim_z <- addinput$dim_z
      X_quant <- addinput$X_quant
      X_qual <- addinput$X_qual
      Y <- addinput$Y
      eps_i <- addinput$eps_i
      k <- addinput$k
      M <- addinput$M
      lb <- addinput$lb
      ub <- addinput$ub
      opt_ctrl <- addinput$opt_ctrl
      x_sol_ele <- stats::optim(x0_ele, objfun, gr = NULL, p_quant, p_qual, lvs_qual, n_lvs_qual,
                                dim_z, X_quant, X_qual, Y, eps_i, k, M,
                                method = 'L-BFGS-B', lower = lb, upper = ub, control = opt_ctrl)
      return(x_sol_ele)
    }
    addinput <- list(p_quant = p_quant, p_qual = p_qual, lvs_qual = lvs_qual, n_lvs_qual = n_lvs_qual,
                     dim_z = dim_z, X_quant = X_quant, X_qual = X_qual, Y = Y, eps_i = NULL,
                     k = k, M = M, lb = lb_ini, ub = ub_ini, opt_ctrl = opt_ctrl_ini)
  }
  # prepare parallel end
  n_try <- length(eps)
  optim_hist <- list()
  #optim_hist$hyper0[[1]] <- hyper0
  for (i_try in 1:n_try) {
    eps_i <- eps[i_try]
    n_opt_i <- nrow(hyper0)
    hyper_sol <- matrix(0, n_opt_i, n_hyper)
    obj_sol <- matrix(0, n_opt_i, 1)
    flag_sol <- matrix(0, n_opt_i, 1)
    # note: could shrink the search space at first step, then expand
    if (parallel) {
      if (i_try == 2) {
        addinput$opt_ctrl <- opt_ctrl_lat
        addinput$lb <- lb_lat
        addinput$ub <- ub_lat
      }
      clust <- makeCluster(min(n_cores,n_opt_i))
      hyper0_list <- list()
      for (j in 1: n_opt_i) {
        hyper0_list[[j]] <- hyper0[j, ]
      }
      addinput$eps_i <- eps_i
      temp_list <- parLapply(clust, hyper0_list, optimfun, neg_log_l, addinput)
      stopCluster(clust)
      for (j in 1: n_opt_i){
        hyper_sol[j, ] <- temp_list[[j]]$par
        obj_sol[j] <- temp_list[[j]]$value
        flag_sol[j] <- temp_list[[j]]$convergence
      }
    } else {
      for (j in 1: n_opt_i){
        if (i_try == 1) {
          temp <- stats::optim(hyper0[j, ], neg_log_l, gr = NULL, p_quant, p_qual, lvs_qual, n_lvs_qual,
                               dim_z, X_quant, X_qual, Y, eps_i, k, M,
                               method = 'L-BFGS-B', lower = lb_ini, upper = ub_ini, control = opt_ctrl_ini)
        } else {
          temp <- stats::optim(hyper0[j, ], neg_log_l, gr = NULL, p_quant, p_qual, lvs_qual, n_lvs_qual,
                               dim_z, X_quant, X_qual, Y, eps_i, k, M,
                               method = 'L-BFGS-B', lower = lb_lat, upper = ub_lat, control = opt_ctrl_lat)
        }
        hyper_sol[j, ] <- temp$par
        obj_sol[j] <- temp$value
        flag_sol[j] <- temp$convergence
      }
    }
    ID = sort(obj_sol, index.return=TRUE)$ix
    obj_sol <- as.matrix(obj_sol[ID, drop = FALSE])
    flag_sol <- as.matrix(flag_sol[ID, drop = FALSE])
    hyper_sol <- as.matrix(hyper_sol[ID, , drop = FALSE])
    hyper0 <- as.matrix(hyper0[ID, , drop = FALSE])

    rownames(obj_sol) <- 1:n_opt_i
    obj_sol_unique  <-  as.matrix(unique(round(obj_sol, 2)))
    flag_sol_unique <- as.matrix(as.data.frame(flag_sol)[c(rownames(obj_sol_unique)), ])
    hyper_sol_unique <- as.matrix(as.data.frame(hyper_sol)[c(rownames(obj_sol_unique)), ])

    #optim_hist$nug_opt[[i]] <- nug_opt
    #optim_hist$raw_min_eig[[i]] <-  raw_min_eig
    optim_hist$hyper0[[i_try]] <- hyper0
    optim_hist$hyper_sol[[i_try]] <-  hyper_sol
    optim_hist$hyper_sol_unique[[i_try]] <-  hyper_sol_unique
    optim_hist$obj_sol[[i_try]] <-  obj_sol
    optim_hist$obj_sol_unique[[i_try]] <- obj_sol_unique
    optim_hist$obj_sol_best[[i_try]] <-  obj_sol[1]
    optim_hist$flag_sol_unique[[i_try]] <- flag_sol_unique

    if (i_try < n_try) {
      hyper0 <- hyper_sol_unique
    }

  }

  fit_time <- proc.time()[3] - start_time


  ## post-processing
  if (!noise) {
    id_best_try <- n_try
  } else {
    id_best_try <- which.min(optim_hist$obj_sol_best)
  }
  hyper_full <- as.matrix(optim_hist$hyper_sol[[id_best_try]][1, ])
  min_n_log_l <- min(optim_hist$obj_sol_best[[id_best_try]])

  if (p_quant == 0) {
    phi <- NULL
  } else {
    phi <- hyper_full[1:p_quant]
  }
  if (p_qual == 0) {
    z_vec <- NULL
    z <- NULL
  } else {
    z_vec <- hyper_full[-c(1:p_quant)]
    z <- list() # z is a list of matrices corresponding to each qualitative variable
    ind_temp <- as.integer(0)
    for (i in 1:p_qual) {
      n_lvs <- n_lvs_qual[i]
      z_i <- z_vec[(dim_z*(ind_temp)+1) : (dim_z*(ind_temp+n_lvs-1))]
      ind_temp <- ind_temp+n_lvs-1
      mat_tmp <- matrix(0, nrow = n_lvs, ncol = dim_z)
      mat_tmp[-1, ] <- matrix(z_i, nrow = n_lvs-1, byrow = TRUE)
      z[[i]] <- mat_tmp
    }
  }


  # calc convenient quantities (stored in model$fit_detail)
  if (p_qual == 0) { # no qualitative/categorical variables
    R <- corr_mat(X_quant, X_quant, phi)
    X_full <- X_quant
  } else {
    X_qual_trans <- to_latent(X_qual, lvs_qual, n_lvs_qual, p_qual, z_vec, dim_z, k)
    X_full <- cbind(X_quant, X_qual_trans)
    omega <- c(phi, rep(0, p_qual*dim_z))
    R <- corr_mat(X_full, X_full, omega)
  }
  R <- (R + t(R))/2

  raw_min_eig = min(eigen(R, symmetric = TRUE, only.values = TRUE)$values)
  if (raw_min_eig < eps[id_best_try]) {
    nug_opt <- eps[id_best_try] - raw_min_eig
    R <- R + diag(x = 1, k, k)*nug_opt
  } else {
    nug_opt <- 0
  }

  L <- t(chol(R)) # R = L%*%t(L)
  Linv <- solve(L)
  LinvM <- solve(L, M)
  MTRinvM <- sum(LinvM^2)
  beta_hat <- t(LinvM)%*%solve(L, Y)/MTRinvM
  beta_hat <- as.numeric(beta_hat)
  temp <- solve(L, Y - M*beta_hat)
  sigma2 <- sum(temp^2)/k
  if ( sigma2 < 1e-300) {
    sigma2 <- 1e-300
  }
  RinvPYminusMbetaP <- solve(t(L), temp)

  ## Save the fitted model
  model <- NULL
  model$quant_param <- list('phi' = phi, 'lb_phi_ini' = lb_phi_ini, 'ub_phi_ini' = ub_phi_ini,
                            'lb_phi_lat' = lb_phi_lat, 'ub_phi_lat' = ub_phi_lat)
  model$qual_param <-  list('dim_z' = dim_z, 'z' = z, 'z_vec' = z_vec,
                            'lb_z' = lb_z, 'ub_z' = ub_z)
  model$data <- list('X' = X, 'X_quant' = X_quant, 'X_qual' = X_qual, 'X_full' = X_full, 'Y' = Y,
                     'X_quant_min' = X_quant_min, 'X_quant_max' = X_quant_max,
                     'Y_min' = Y_min, 'Y_max' = Y_max,
                     'ind_qual' = ind_qual, 'lvs_qual' = lvs_qual, 'n_lvs_qual' = n_lvs_qual,
                     'p_all' = p_all, 'p_quant' = p_quant, 'p_qual' = p_qual)
  model$fit_detail <- list('beta_hat' = beta_hat, 'sigma2'=sigma2, 'MTRinvM' = MTRinvM,
                           'Linv' = Linv, 'LinvM' = LinvM, 'RinvPYminusMbetaP'=RinvPYminusMbetaP,
                           'raw_min_eig' = raw_min_eig,
                           'nug_opt' = nug_opt, 'min_n_log_l' = min_n_log_l, 'fit_time' = fit_time)
  model$optim_hist <- optim_hist
  model$setting <- setting
  class(model) <- 'LVGP model'

  return(model)
}

#' @title The Prediction Function of \code{LVGP} Package
#' @description Predicts the output and associated uncertainties of the GP model fitted by \code{\link[LVGP]{LVGP_fit}}.
#'
#' @param X_new Matrix or vector containing the input(s) where the predictions are to be made. Each row is an input vector.
#' @param model The LVGP model fitted by \code{\link[LVGP]{LVGP_fit}}.
#' @param MSE_on A scalar indicating whether the uncertainty (i.e., mean squared error \code{MSE}) is calculated.
#'     Set to a non-zero value to calculate \code{MSE}.
#'
#' @return A prediction list containing the following components:
#' \describe{
#' \item{Y_hat}{A vector containing the mean prediction values}
#' \item{MSE}{A vector containing the prediction uncertainty (i.e., the covariance or covariance matrix for the output(s) at prediction location(s))}
#' }
#'
#' @export
#'
#' @references
#' \enumerate{
#' \item "A Latent Variable Approach to Gaussian Process Modeling with Qualitative and Quantitative Factors", Yichi Zhang, Siyu Tao, Wei Chen, and Daniel W. Apley (\href{https://arxiv.org/abs/1806.07504}{arXiv})
#' }
#'
#' @seealso
#' \code{\link[LVGP]{LVGP_fit}} to fit LVGP model to the datasets.\cr
#' \code{\link[LVGP]{LVGP_plot}} to plot the features of the fitted model.
#'
#' @examples
#' # see the examples in the documentation of the function LVGP_fit.

LVGP_predict <-  function(X_new, model, MSE_on = 0){

  ## import model and check inputs
  if (!inherits(model, "LVGP model")){
    stop('    The 2nd input should be a model of class "LVGP model".')
  }
  if (length(MSE_on)!=1){
    stop('    MSE_on should be a scalar flag. Set it to 1 to turn it "on".')
  }
  if (is.list(X_new)) {
    stop('    The package do not yet support "list" type for X_new.')
  }
  p_all <- model$data$p_all
  if (is.null(nrow(X_new))) {
    if (length(X_new) != p_all){
      stop('    The dimensionality of X_new is not correct!')
    }
  } else {
    if (ncol(X_new) != p_all){
      stop('    The dimensionality of X_new is not correct!')
    }
  }
  p_qual <- model$data$p_qual
  #p_quant <- model$data$p_quant

  X_quant_min <- model$data$X_quant_min
  X_quant_max <- model$data$X_quant_max
  Y_min <- model$data$Y_min
  Y_max <- model$data$Y_max

  X_old_full <- model$data$X_full
  lvs_qual <- model$data$lvs_qual
  n_lvs_qual <- model$data$n_lvs_qual
  ind_qual <- model$data$ind_qual

  phi <- model$quant_param$phi
  dim_z <- model$qual_param$dim_z
  z_vec <- model$qual_param$z_vec

  beta_hat <- as.numeric(model$fit_detail$beta_hat)
  RinvPYminusMbetaP <- model$fit_detail$RinvPYminusMbetaP

  ## process X_new
  if (is.null(nrow(X_new))) {
    m <- 1
    X_new <- matrix(X_new, 1, p_all)
    X_new_2 <- as.data.frame(rbind(X_new, X_new)) # convert to dataframe
    X_new <- X_new_2[1,]
  } else {
    m <- nrow(X_new)
  }

  if (p_qual == 0) { # no qualitative/categorical variables
    X_new_qual <- NULL
    X_new_quant <- X_new
    if (is.matrix(X_new_quant) == FALSE) {
      X_new_quant <- as.matrix(X_new_quant)
    }
    if (!all(is.finite(X_new_quant)) || !is.numeric(X_new_quant)){
      stop('    All the elements of X_new_quant must be finite numbers.')
    }
    X_new_quant <- t((t(X_new_quant)-X_quant_min)/(X_quant_max-X_quant_min))

    R_old_new <- corr_mat(X_old_full, X_new_quant, phi)
    R_new_new <- corr_mat(X_new_quant, X_new_quant, phi)
  } else {
    X_new_qual <- X_new[, ind_qual]
    if (is.data.frame(X_new_qual) == FALSE) {
      X_new_qual <- as.data.frame(X_new_qual)
    }

    if (p_qual == p_all) { # all the variables are qualitative/categorical
      X_new_quant <- NULL
    } else {
      X_new_quant <- X_new[, -ind_qual] # data of quant variables
      if (is.matrix(X_new_quant) == FALSE) {
        X_new_quant <- as.matrix(X_new_quant)
      }
      if (!all(is.finite(X_new_quant)) || !is.numeric(X_new_quant)){
        stop('    All the elements of X_new_quant must be finite numbers.')
      }
      X_new_quant <- t((t(X_new_quant)-X_quant_min)/(X_quant_max-X_quant_min))
    }

    X_new_qual_la <- to_latent(X_new_qual, lvs_qual, n_lvs_qual, p_qual, z_vec, dim_z, m)
    X_new_full <- cbind(X_new_quant, X_new_qual_la)
    phi_full <- c(phi, rep(0, p_qual*dim_z))
    R_old_new <- corr_mat(X_old_full, X_new_full, phi_full)
    R_new_new <- corr_mat(X_new_full, X_new_full, phi_full)
  }
  R_new_new <- (R_new_new + t(R_new_new))/2

  ## calc predictions
  Y_hat <- beta_hat+t(R_old_new)%*%RinvPYminusMbetaP
  Y_hat = t(t(Y_hat)*(Y_max-Y_min) + Y_min)

  prediction <- list()
  prediction$Y_hat <- Y_hat

  if (MSE_on) {
    # import relevant quantities
    sigma2 <- model$fit_detail$sigma2
    Linv <- model$fit_detail$Linv
    MTRinvM <- model$fit_detail$MTRinvM
    LinvM <- model$fit_detail$LinvM

    temp <- Linv%*%R_old_new
    W <- 1-t(LinvM)%*%temp
    MSE <- sigma2*(R_new_new-t(temp)%*%temp+t(W)%*%W/MTRinvM)*((Y_max-Y_min)^2)
    prediction$MSE <- MSE
  }

  return(prediction)
}

#' @title The Plotting Function of \code{LVGP} Package
#'
#' @description Plots the qualitative/categorical variable levels in the latent space (only for 1D or 2D cases).
#'     If the qualitative/categorical variables are not specified, all the qualified variables will be plotted.
#'     See \code{Arguments} for more details on the options.
#'
#' @param model The LVGP model fitted by \code{\link[LVGP]{LVGP_fit}}
#' @param ind_qual_plot An array of index (indices) of the qualitative/categorical variable(s) to be plotted.
#'     Default is NULL, in which case all the qualitative/categorical variables will be plotted.
#'
#' @return NULL
#'
#' @import graphics
#'
#' @note This plot function \strong{only} works for 1D or 2D latent spaces.
#' @export
#'
#' @references
#' \enumerate{
#' \item "A Latent Variable Approach to Gaussian Process Modeling with Qualitative and Quantitative Factors", Yichi Zhang, Siyu Tao, Wei Chen, and Daniel W. Apley (\href{https://arxiv.org/abs/1806.07504}{arXiv})
#' }
#'
#' @seealso
#' \code{\link[LVGP]{LVGP_fit}} to fit LVGP model to the datasets.\cr
#' \code{\link[LVGP]{LVGP_predict}} to use the fitted LVGP model for prediction.
#'
#' @examples
#' # see the examples in the documentation of the function LVGP_fit.

LVGP_plot <- function(model, ind_qual_plot = NULL){
  ind_qual_all <- model$data$ind_qual

  ## checking indices of the variables to be plotted
  if (is.null(ind_qual_all)) {
    stop('    No qualitative/categorical variables in the model. No plot.')
  }
  if (is.null(ind_qual_plot)) {
    ind_qual_plot <- ind_qual_all
    n_plot <- length(ind_qual_plot)
    qual_only_ind <- 1:n_plot
  } else {
    if (!is.numeric(ind_qual_plot)) {
      stop('    The specified index (indices) are invalid!')
    }
    ind_qual_plot <- as.vector(as.integer(ind_qual_plot))
    n_init <- length(ind_qual_plot)
    ind_qual_plot <- unique(ind_qual_plot)
    n_uniq <- length(ind_qual_plot)
    if (n_init != n_uniq) {
      warning('    Ignoring duplicate indices specified.')
    }
    checkers <- (ind_qual_plot %in% ind_qual_all)
    if (!all(checkers)) {
      warning('    Ignoring invalid indices.')
    }
    n_plot <- sum(checkers)
    if (n_plot == 0) {
      stop('    All indices input are invalid! No plot.')
    }
    ind_qual_plot <- ind_qual_plot[checkers]
    qual_only_ind <- match(ind_qual_plot, ind_qual_all)
  }

  ## loading more data
  dim_z <- model$qual_param$dim_z
  z <- model$qual_param$z
  lvs_qual <- model$data$lvs_qual

  ## plotting
  for (i in 1:n_plot) {
    qual_only_ind_i <- qual_only_ind[i]
    z_i <- z[[qual_only_ind_i]]
    lvs_qual_i <- signif(lvs_qual[[qual_only_ind_i]], 3)
    if (dim_z == 1) {
      z_i <- as.vector(z_i)
      stripchart(z_i,
                 main = paste('Latent variables for var #', toString(ind_qual_plot[i]), sep = ''),
                 ylab = "", xlab = "Latent variable value")
      text(z_i, 1.1, labels=lvs_qual_i)
    } else if (dim_z == 2) {
      x_range <- max(z_i[,1]) - min(z_i[,1])
      y_range <- max(z_i[,2]) - min(z_i[,2])
      plot(z_i[,1], z_i[,2], type = "p",
           main = paste('Latent variables for var #', toString(ind_qual_plot[i]), sep = ''),
           ylab = "Latent variable 2 value", xlab = "Latent variable 1 value")
      text(z_i[,1]-x_range*0.02, z_i[,2]-y_range*0.02, labels=lvs_qual_i)
    } else {
      stop('    This plot function only works for 1D or 2D latent spaces!')
    }
  }

}

#' @title The Negative Log-Likelehood Function in \code{LVGP} Package
#'
#' @description Calculates the negative log-likelihood (excluding all the constant terms) as described in \code{reference 1}.
#'
#' @param hyperparam Hyperparameters of the LVGP model
#' @param p_quant Number of quantative variables
#' @param p_qual Number of qualitative variables
#' @param lvs_qual Levels of each qualitative variable
#' @param n_lvs_qual Number of levels of each qualitative variable
#' @param dim_z Dimensionality of latent variables, usually 1 or 2
#' @param X_quant Input data of quantative variables
#' @param X_qual Input data of qualitative variables
#' @param Y Vector containing the outputs of data points
#' @param min_eig The smallest eigen value that the correlation matrix is allowed to have, which determines the nugget added to the correlation matrix.
#' @param k Number of data points, \code{nrow(X_quant)} or \code{nrow(X_qual)}
#' @param M Vector of ones with length \code{k}
#'
#' @return The negative log-likelihood (excluding all the constant terms) value.
#'
#' @details \code{\link[LVGP]{LVGP_fit}} calls this function as its optimization objective function.
#'
#' @note This function is \strong{NOT} exported once the package is loaded.
#' @export
#'
#' @references
#' \enumerate{
#' \item "A Latent Variable Approach to Gaussian Process Modeling with Qualitative and Quantitative Factors", Yichi Zhang, Siyu Tao, Wei Chen, and Daniel W. Apley (\href{https://arxiv.org/abs/1806.07504}{arXiv})
#' }
#'
#' @seealso
#' \code{\link[LVGP]{LVGP_fit}} to see how a GP model can be fitted to a training dataset.\cr
#' \code{\link[LVGP]{LVGP_predict}} to use the fitted LVGP model for prediction.\cr
#' \code{\link[LVGP]{LVGP_plot}} to plot the features of the fitted model.
#'
#' @examples
#' # see the examples in the documentation of the function LVGP_fit.

neg_log_l <- function(hyperparam, p_quant, p_qual, lvs_qual, n_lvs_qual, dim_z,
                      X_quant, X_qual, Y, min_eig, k, M){
  if (p_qual == 0) { # no qualitative/categorical variables
    R <- corr_mat(X_quant, X_quant, hyperparam)
  } else {
    z_vec <- hyperparam[-c(1:p_quant)]
    X_qual_la <- to_latent(X_qual, lvs_qual, n_lvs_qual, p_qual, z_vec, dim_z, k)
    X_full <- cbind(X_quant, X_qual_la)
    phi_full <- c(hyperparam[1:p_quant], rep(0, p_qual*dim_z))
    R <- corr_mat(X_full, X_full, phi_full)
  }
  R <- (R + t(R))/2

  raw_min_eig = min(eigen(R, symmetric = TRUE, only.values = TRUE)$values)
  if (raw_min_eig < min_eig){
    R = R + diag(x = 1, k, k)*(min_eig - raw_min_eig)
  }

  L <- t(chol(R)) # R = L%*%t(L)
  LinvM <- solve(L, M)
  beta_hat <- t(LinvM)%*%solve(L, Y)/sum(LinvM^2)
  beta_hat <- as.numeric(beta_hat)
  temp <- solve(L, Y - M*beta_hat)
  sigma2 <- sum(temp^2)/k
  if ( sigma2 < 1e-300) {
    sigma2 <- 1e-300
  }

  det_R <- det(R)
  if (det_R < 1e-300) {
    det_R <- 1e-300
  }
  n_log_l <- k*log(sigma2) + log(det_R)

  return(n_log_l)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/LVGP_fit.R
\name{LVGP_fit}
\alias{LVGP_fit}
\title{The Fitting Function of \code{LVGP} Package}
\usage{
LVGP_fit(X, Y, ind_qual = NULL, dim_z = 2, eps = 10^(seq(-1, -8,
  length.out = 15)), lb_phi_ini = -2, ub_phi_ini = 2,
  lb_phi_lat = -8, ub_phi_lat = 3, lb_z = -3, ub_z = 3,
  n_opt = 8, max_iter_ini = 100, max_iter_lat = 20, seed = 123,
  progress = FALSE, parallel = FALSE, noise = FALSE)
}
\arguments{
\item{X}{Matrix or data frame containing the inputs of training data points. Each row is a data point.}

\item{Y}{Vector containing the outputs of training data points}

\item{ind_qual}{Vector containing the indices of columns of qualitative/categorical variables}

\item{dim_z}{Dimensionality of latent space, usually 1 or 2 but can be higher}

\item{eps}{The vector of smallest eigen values that the correlation matrix is allowed to have, which determines the nugget added to the correlation matrix.}

\item{lb_phi_ini, ub_phi_ini}{The initial lower and upper search bounds of the scale/roughness parameters (\code{phi}) of quantitative variables}

\item{lb_phi_lat, ub_phi_lat}{The later lower and upper search bounds of the scale/roughness parameters (\code{phi}) of quantitative variables}

\item{lb_z, ub_z}{The lower and upper search bounds of the latent parameters (\code{z}) of qualitative variables}

\item{n_opt}{The number of times the log-likelihood function is optimized}

\item{max_iter_ini}{The maximum number of iterations for each optimization run for largest (first) eps case}

\item{max_iter_lat}{The maximum number of iterations for each optimization run for after first eps cases}

\item{seed}{An integer for the random number generator. Use this to make the results reproducible.}

\item{progress}{The switch determining whether to print function run details}

\item{parallel}{The switch determining whether to use parallel computing}

\item{noise}{The switch for whether the data are assumed noisy}
}
\value{
A model of class \code{"LVGP model"}, a list with the following components:
\describe{
\item{quant_param}{A list containing the estimated parameter \code{phi} and its search bounds for quantitative variables}
\item{qual_param}{A list containing the estimated parameter \code{z} and its dimensionality, vectorized form and search bounds for qualitative variables}
\item{data}{A list containing the fitted dataset in verbose format}
\item{fit_detail}{A list of more detailed variables for fitting and prediction process}
\item{optim_hist}{Optimization history}
\item{setting}{Settings for the optimization and fitting process}
}
}
\description{
Fits a latent-variable Gaussian process (LVGP) model to a dataset as described in \code{reference 1}.
    The input variables can be quantitative or qualitative/categorical or mixed.
    The output variable is quantitative and scalar.
}
\examples{
# Math example with 2 quantitative and 1 qualitative variables (dataset included in the package):
#     Fit a model (with default settings) and evaluate the performance
#     by computing the root mean squared error (RMSE) in prediction.
#     Also, plot the latent variable parameters.
X_tr <- math_example$X_tr
Y_tr <- math_example$Y_tr
X_te <- math_example$X_te
Y_te <- math_example$Y_te
n_te <- nrow(X_te)
model <- LVGP_fit(X_tr, Y_tr, ind_qual = c(3))
output <- LVGP_predict(X_te, model)
Y_hat <- output$Y_hat
RRMSE <- sqrt(sum((Y_hat-Y_te)^2)/n_te)/(max(Y_te)-min(Y_te))
LVGP_plot(model)

}
\references{
\enumerate{
\item "A Latent Variable Approach to Gaussian Process Modeling with Qualitative and Quantitative Factors", Yichi Zhang, Siyu Tao, Wei Chen, and Daniel W. Apley (\href{https://arxiv.org/abs/1806.07504}{arXiv})
}
}
\seealso{
\code{\link[stats]{optim}} for the details on \code{L-BFGS-B} algorithm used in optimization.\cr
\code{\link[LVGP]{LVGP_predict}} to use the fitted LVGP model for prediction.\cr
\code{\link[LVGP]{LVGP_plot}} to plot the features of the fitted model.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/LVGP_predict.R
\name{LVGP_predict}
\alias{LVGP_predict}
\title{The Prediction Function of \code{LVGP} Package}
\usage{
LVGP_predict(X_new, model, MSE_on = 0)
}
\arguments{
\item{X_new}{Matrix or vector containing the input(s) where the predictions are to be made. Each row is an input vector.}

\item{model}{The LVGP model fitted by \code{\link[LVGP]{LVGP_fit}}.}

\item{MSE_on}{A scalar indicating whether the uncertainty (i.e., mean squared error \code{MSE}) is calculated.
Set to a non-zero value to calculate \code{MSE}.}
}
\value{
A prediction list containing the following components:
\describe{
\item{Y_hat}{A vector containing the mean prediction values}
\item{MSE}{A vector containing the prediction uncertainty (i.e., the covariance or covariance matrix for the output(s) at prediction location(s))}
}
}
\description{
Predicts the output and associated uncertainties of the GP model fitted by \code{\link[LVGP]{LVGP_fit}}.
}
\examples{
# see the examples in the documentation of the function LVGP_fit.
}
\references{
\enumerate{
\item "A Latent Variable Approach to Gaussian Process Modeling with Qualitative and Quantitative Factors", Yichi Zhang, Siyu Tao, Wei Chen, and Daniel W. Apley (\href{https://arxiv.org/abs/1806.07504}{arXiv})
}
}
\seealso{
\code{\link[LVGP]{LVGP_fit}} to fit LVGP model to the datasets.\cr
\code{\link[LVGP]{LVGP_plot}} to plot the features of the fitted model.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/LVGP_plot.R
\name{LVGP_plot}
\alias{LVGP_plot}
\title{The Plotting Function of \code{LVGP} Package}
\usage{
LVGP_plot(model, ind_qual_plot = NULL)
}
\arguments{
\item{model}{The LVGP model fitted by \code{\link[LVGP]{LVGP_fit}}}

\item{ind_qual_plot}{An array of index (indices) of the qualitative/categorical variable(s) to be plotted.
Default is NULL, in which case all the qualitative/categorical variables will be plotted.}
}
\description{
Plots the qualitative/categorical variable levels in the latent space (only for 1D or 2D cases).
    If the qualitative/categorical variables are not specified, all the qualified variables will be plotted.
    See \code{Arguments} for more details on the options.
}
\note{
This plot function \strong{only} works for 1D or 2D latent spaces.
}
\examples{
# see the examples in the documentation of the function LVGP_fit.
}
\references{
\enumerate{
\item "A Latent Variable Approach to Gaussian Process Modeling with Qualitative and Quantitative Factors", Yichi Zhang, Siyu Tao, Wei Chen, and Daniel W. Apley (\href{https://arxiv.org/abs/1806.07504}{arXiv})
}
}
\seealso{
\code{\link[LVGP]{LVGP_fit}} to fit LVGP model to the datasets.\cr
\code{\link[LVGP]{LVGP_predict}} to use the fitted LVGP model for prediction.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/to_latent.R
\name{to_latent}
\alias{to_latent}
\title{The Function for Transforming Qualitative/Categorical Variables into Latent Variables in \code{LVGP} Package}
\usage{
to_latent(X_qual, lvs_qual, n_lvs_qual, p_qual, z_vec, dim_z, k)
}
\arguments{
\item{X_qual}{Matrix or data frame containing (only) the qualitative/categorical data.}

\item{lvs_qual}{List containing levels of each qualitative variable}

\item{n_lvs_qual}{Number of levels of each qualitative variable}

\item{p_qual}{Number of qualitative variables}

\item{z_vec}{Latent variable parameters, i.e., latent variable values for each level of qualitative/categorical variables}

\item{dim_z}{Dimensionality of latent variables, usually 1 or 2}

\item{k}{Number of data points, equal to \code{nrow(X_qual)}}
}
\value{
Matrix containing transformed data
}
\description{
Transforms qualitative/categorical variables into latent variables.
}
\note{
This function is \strong{NOT} exported once the LVGP package is loaded.
}
\examples{
# see the examples in the documentation of the function LVGP_fit.
}
\references{
\enumerate{
\item "A Latent Variable Approach to Gaussian Process Modeling with Qualitative and Quantitative Factors", Yichi Zhang, Siyu Tao, Wei Chen, and Daniel W. Apley (\href{https://arxiv.org/abs/1806.07504}{arXiv})
}
}
\seealso{
\code{\link[LVGP]{LVGP_fit}} to see how a GP model can be fitted to a training dataset.\cr
\code{\link[LVGP]{LVGP_predict}} to use the fitted LVGP model for prediction.\cr
\code{\link[LVGP]{LVGP_plot}} to plot the features of the fitted model.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/neg_log_l.R
\name{neg_log_l}
\alias{neg_log_l}
\title{The Negative Log-Likelehood Function in \code{LVGP} Package}
\usage{
neg_log_l(hyperparam, p_quant, p_qual, lvs_qual, n_lvs_qual, dim_z,
  X_quant, X_qual, Y, min_eig, k, M)
}
\arguments{
\item{hyperparam}{Hyperparameters of the LVGP model}

\item{p_quant}{Number of quantative variables}

\item{p_qual}{Number of qualitative variables}

\item{lvs_qual}{Levels of each qualitative variable}

\item{n_lvs_qual}{Number of levels of each qualitative variable}

\item{dim_z}{Dimensionality of latent variables, usually 1 or 2}

\item{X_quant}{Input data of quantative variables}

\item{X_qual}{Input data of qualitative variables}

\item{Y}{Vector containing the outputs of data points}

\item{min_eig}{The smallest eigen value that the correlation matrix is allowed to have, which determines the nugget added to the correlation matrix.}

\item{k}{Number of data points, \code{nrow(X_quant)} or \code{nrow(X_qual)}}

\item{M}{Vector of ones with length \code{k}}
}
\value{
The negative log-likelihood (excluding all the constant terms) value.
}
\description{
Calculates the negative log-likelihood (excluding all the constant terms) as described in \code{reference 1}.
}
\details{
\code{\link[LVGP]{LVGP_fit}} calls this function as its optimization objective function.
}
\note{
This function is \strong{NOT} exported once the package is loaded.
}
\examples{
# see the examples in the documentation of the function LVGP_fit.
}
\references{
\enumerate{
\item "A Latent Variable Approach to Gaussian Process Modeling with Qualitative and Quantitative Factors", Yichi Zhang, Siyu Tao, Wei Chen, and Daniel W. Apley (\href{https://arxiv.org/abs/1806.07504}{arXiv})
}
}
\seealso{
\code{\link[LVGP]{LVGP_fit}} to see how a GP model can be fitted to a training dataset.\cr
\code{\link[LVGP]{LVGP_predict}} to use the fitted LVGP model for prediction.\cr
\code{\link[LVGP]{LVGP_plot}} to plot the features of the fitted model.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/corr_mat.R
\name{corr_mat}
\alias{corr_mat}
\title{The Function for Constructing the Correlation Matrix in \code{LVGP} Package}
\usage{
corr_mat(X1, X2, phi_full)
}
\arguments{
\item{X1, X2}{Matrices containing the data points. The rows and columns of both \code{X1} and \code{X2} denote individual observation settings and dimension, respectively.}

\item{phi_full}{The vector storing all the scale (aka roughness) parameters of the correlation function. See \code{reference 1}.}
}
\value{
R The Correlation matrix with size \code{nrow(X1)}-by-\code{nrow(X2)}. See \href{https://en.wikipedia.org/wiki/Correlation_matrix}{here}.
}
\description{
Builds the correlation matrix given two datasets, and the type and parameters of the correlation function.
}
\note{
This function is \strong{NOT} exported once the LVGP package is loaded.
}
\examples{
# see the examples in the documentation of the function LVGP_fit.
}
\references{
\enumerate{
\item "A Latent Variable Approach to Gaussian Process Modeling with Qualitative and Quantitative Factors", Yichi Zhang, Siyu Tao, Wei Chen, and Daniel W. Apley (\href{https://arxiv.org/abs/1806.07504}{arXiv})
}
}
\seealso{
\code{\link[LVGP]{LVGP_fit}} to see how a GP model can be fitted to a training dataset.\cr
\code{\link[LVGP]{LVGP_predict}} to use the fitted LVGP model for prediction.\cr
\code{\link[LVGP]{LVGP_plot}} to plot the features of the fitted model.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/example-data.R
\docType{data}
\name{math_example}
\alias{math_example}
\title{Dataset for the example in function 'LVGP_fit'}
\format{A named list containing training and test data:
\describe{
  \item{"X_tr"}{24-by-3 matrix for 24 training data inputs, 3rd column being the qualitative variable}
  \item{"Y_tr"}{24-by-1 matrix for 24 training data outputs}
  \item{"X_te"}{90-by-3 matrix for 90 testing data inputs, 3rd column being the qualitative variable}
  \item{"Y_te"}{90-by-1 matrix for 90 testing data outputs}
}}
\source{
The dataset can be generated with the code at the end of this description file.
}
\usage{
data(math_example)
}
\description{
Data are sampled from the modified math function based on the first example in the paper listed in \code{references}.
There are still 2 quantitative and 1 qualitative variables, but the qualitative variable has only 3 levels.
For each level, there are 8 training data points and 30 testing data points, all generated with Latin hypercube sampling.
In total, there are 24 training data points and 90 testing data points.
}
\examples{
data(math_example)
X_tr <- math_example$X_tr
Y_tr <- math_example$Y_tr
X_te <- math_example$X_te
Y_te <- math_example$Y_te
}
\references{
\enumerate{
\item "A Latent Variable Approach to Gaussian Process Modeling with Qualitative and Quantitative Factors", Yichi Zhang, Siyu Tao, Wei Chen, and Daniel W. Apley (\href{https://arxiv.org/abs/1806.07504}{arXiv})
}
}
\keyword{dataset}
\keyword{example}
